___________________________________________
# Kavach_artifacts
___________________________________________
___________________________________________

Welcome to the `kavach_artifacts/HW` folder of the artifacts! 

The hardware implementation files are present here. The files of interest would be the hardware implementation files stored in the folder ./design_files (download from the repository at https://extgit.iaik.tugraz.at/sesys/kavach_artifacts/-/tree/artifact_v1.2/HW/design_files). The structure of this folder is defined below. Following this, we have also given a brief description of these files.

```
.HW
|
|-----design_files
      |
      |____ComputeCoreWrapper.v
      |____ComputeCore3.v
      |____set_random.v
      |
      |----AOverflowBit
           |____CA_gen.v
           |____A2A.v
           |____parallel_a2a.v
           |____secret_overflow.v
      |
      |----Vector_multiplier
           |____s_mul.v
           |____parallel_Mults.v
           |____buffer_muxer.v
           |____poly_load_control_BRAM.v
           |____pol_mul256_parallel_in.v
           |____VectorMul.v
           |____VectorMul_wrapper.v
      |
      |____set_trigger.v
      |____copy_words.v
      |____sub_poly.v
      |____trivium_new.v, trivium_64coe.v
      |
      |----memory
      |----port_definition
|
|_____cw305_main.xdc  
|_____cw305_top.bit
|_____kavach.tcl
|_____kavach2022.2.tcl
|_____README.md
```
___________________________________________

## Description:


- `kavach.tcl` can be used to generate the vivado project. For this open Vivado in ./HW/ >> open the Tcl console >> execute $source kavach.tcl. Now Vivado will create project folder called kavach which is ready to use. A fresh bitstream file can also be generated this way. This tcl script is tested for Vivado2019.1. We also provide a new tcl script kavach_2022.2.tcl as some property names change between these two vivado versions. If neither of these two scripts works for the user, the user can just create a new project for Artix-7, export the design files, and generate a fresh bitstream.


- `cw305_main.xdc` is the vivado constraint file.

- `cw305_top.bit` is the generated bitstream file.


In folder design_files:

- `ComputeCoreWrapper.v`: This is the top file of our design, and it interacts with the block design. It instantiates the core module ComputeCore3, which contains all the masked designs and other helper building blocks. This module also contains an Instruction controller that reads the instructions and requests ComputeCore3 to process them. As soon as it receives a done signal, it reads another instruction and sends a processing request.

- `ComputeCore3.v`: This file contains activation and instantiations of all the building blocks namely: 1. VectorMul_wrapper, 2. AOverflowBit(Secret_Overflow), 3. SETTRIGGER, 4. Trivium, 5. copy_words, 6. sub_poly, 7. set_random, and 8. BRAM.

- > `AOverflowBit/CA_gen.v`: This file contains an instantiation of the C_A table given in equation 1 of the paper. 
- > `AOverflowBit/A2A.v`: This file contains an instantiation of the AOverflowBit function described in Algorithm 3 of the paper. 
- > `AOverflowBit/parallel_a2a.v`: This module instantiates 16 AOverflowBit modules because one 64-bit word read from the address contains 16 secret coefficients. So, we process them in parallel. 
- > `AOverflowBit/secret_overflow.v`: This file contains the wrapper for reading and writing the correct secret shares. Memory word size is 64-bit, but 'c_0,c_1' bits generated by processing twp memory words full of secret coefficient shares are only 16 per share. Thus, we wait for four such results before writing the fully packed memory words.


- > `Vector_multiplier/s_mul.v`: This file contains the instantiation of MskMux and HybridLin given in Algorithm 5 and Algorithm 6 of the paper. 
- > `Vector_multiplier/parallel_Mults.v`: This files instantaites 256 parallel s_mul modules for a high-speed polynomial multiplication.
- > `Vector_multiplier/buffer_muxer.v`: This module fetches the correct public polynomial (a)'s coefficient from the buffer. More information about this can be found in the paper [RB20]. 
- > `Vector_multiplier/poly_load_control_BRAM.v`: This file controls the polynomial reads for the polynomial multiplication. 
- > `Vector_multiplier/pol_mul256_parallel_in.v`: This is the wrapper file on polynomial multiplication that controls the buffers for secret and public polynomial storage and the polynomial multiplication iterations.
- > `Vector_multiplier/VectorMul.v`: This file contains a controller to ensure that a full polynomial vector multiplication is finished. 
- > `Vector_multiplier/VectorMul_wrapper.v`: This file is a wrapper on all the modules and connects the vector multiplication controller (VectorMul.v) to the polynomial multiplication unit(pol_mul256_parallel_in.v). 


- `set_trigger.v`: This function starts the trigger and waits for 8000 clock cycles for the noise from this high-voltage peak to die. For end_trigger, this is done only for three clock cycles. 

- `set_random.v`: This file is used to initialize the randomness for the AOverflowBit module. It is kept separate so that it does not change between the static and random case instructions. 

- `copy_words.v`: This is used to copy words from one memory address to another. It is used during the measurements collection so that both the static and random cases work with the same memory addresses. 

- `sub_poly.v`: This is used to subtract chunks of 4 bits (the size of secret coefficients). It is used during the measurements collection for the static case to make arithmetic shares of the secret coefficients. 

- `trivium_new.v, trivium_64coe.v`: This is used as a pseudo-random number generator. It generates a 64-bit random value every clock cycle, and is mainly used for refreshing the randomness in the BRAM for static-vs-random t-test evaluation. C-code reference available at https://github.com/cbouilla/trivium .

- The `memory` and `port_definition` folders contain memory IP and port definiitons.



________________________________________________________________________

