___________________________________________
___________________________________________
# Kavach_artifacts
___________________________________________
___________________________________________

Welcome to the `kavach_artifacts/SW` folder of the artifacts! 

This folder contains files that help us run the instructions, verify them, and collect measurements. We send instructions as this is an instruction-set architecture. 

```
.SW
|
|_____crypto_setup_masked_design.py
|
|_____crypto_setup_masked_flawed_design.py
|
|_____crypto_memory_dump.py
|
|_____verify_masked_design.py
|
|_____store_raw_data_of_measurement.py
|
|_____README.md
```
___________________________________________

## Description:

- Files `crypto_setup_masked_design.py` and `crypto_setup_flawed_masked_design.py` send the instruction given below to evaluate the static vs random case with good/bad randomness. `How this works', is explained at the end of this Readme.

- File `crypto_memory_dump.py` is used to get the memory dump from the FPGA on chipwhisperer. This is used to verify if the final result is the same as the expected result and if the operations ran correctly.

- File `verify_masked_design.py` is used to automate the verification of AOverflowBit followed by the polynomial vector multiplication. 

- File `store_raw_data_of_measurement.py` fetches the voltage and trigger data from the picoscope.


___________________________________________

# Instructions explained

 | INS | Function |
 |-----|---------|
 |006 | Polynomial vector Multiplication|
 011 | Trivium
 012 | Copy
 020 | AOverflowBit
 021 | Polynomial vector Multiplication
 022 | Polynomial vector Multiplication
 025 | Subtraction
 026 | Start Trigger
 027 | Stop Trigger
 028 | Set Gamma and randomness for AOverflowBit
 029 | Wait

 All instructions are instantiated, with four elements,  INS, OP1, OP2, and WT. INS serves the purpose of Instruction code, OP1 and OP2 serve as read addresses, and WT serves as the write address for dyadic operations like subtraction, polynomial vector multiplication, etc. For other instructions example, in the case of Trivium or the Wait instructions, OP2 serves as a counter to tell how many random values to generate or how many clock cycles to wait for.  
 
 The three different multiplication instructions are used to distinguish between Multiplication with secret shares, Multiplication with the addition of 'h'(specific to Saber), and plain Multiplication. 
___________________________________________

# Good vs Bad randomness explained with memory storage

- In the beginning, the first 500 addresses are filled with random values. 

- After this, in every iteration, the Trivium randomizes the first 310 values in case of good randomness and 307 in case of bad randomness. 

- Secret is stored at address 360-407.

 | |`Good randomness`         | `Bad randomness`|
 |-|------------------------|---------------|
 |STATIC case:||
 | | s_0 at 210-257         | s_0 at 310-357 (does not get refreshed by trivium thus, bad randomness)|
 | | s_1 = s-s_0 310-357    | s_1 = s-s_0 at 210-257
 | |AOverflowBit reads s_0 and s_1 and writes at 410-457. Then we perform three different vector multiplication of the public polynomials stored starting from address 2, with each of these secret polynomial shares.|AOverflowBit reads s_0 and s_1 and writes at 410-457. Then we perform three different vector multiplication of the public polynomials stored starting from address 2, with each of these secret polynomial shares.|
| RANDOM case: (same for both good and bad randomness case)||
 |    |s_0 at 160-207  (random value)|s_0 at 160-207  (random value)*|
  |  |s_1 ate 260-307 (random value) |s_1 ate 260-307 (random value)*|
    *We copy the two address 210-257 and 310-357 to 160-207 and 260-307. 

___________________________________________

With these, we conclude the SW description section. For more details on how these instructions are processed, please refer to the HW Readme file and codes. 

___________________________________________
