# Copyright TU Graz, Austria <sujoy.sinharoy@iaik.tugraz.at>.
#
# This file is part of Kavach <https://extgit.iaik.tugraz.at/sesys/kavach_artifacts>.
#
# Kavach is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License version 3 as published by the Free
# Software Foundation.
#
# Kavach is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Kavach. If not, see <https://www.gnu.org/licenses/gpl-3.0.txt>.

import random

def poly_mul(a, s, acc=None,bs=False):
# schoolbook polynomial multiplication is defined here
	acc_temp = [0 for _ in range(2*256)]

		
	for i in range(256):
		for j in range(256):
			s_temp = (s[(int)(i/16)]>>((i%16)*4))&0xf
			a_temp = ((a[(int)(j/4)]>>((j%4)*16))&0x1fff)
			if(bs):
				acc_temp[i+j] = (acc_temp[i+j] +  a_temp*((2**4 * s_temp)+4))% (2**13)
			else:
				acc_temp[i+j] = (acc_temp[i+j] + a_temp*s_temp)% (2**13)
			

	for i in range(256,512):
		acc_temp[i-256] = (acc_temp[i-256]-acc_temp[i]) % (2**13)
	
	#accumulating with the existing results in acc	
	for i in range(0,256):
		acc[i] = (acc[i]+acc_temp[i]) % (2**13)
	

	return acc


def vector_mul(aa, ss, acc_value=0, bs=False):
# this function processes three polynomial multilications and ensures that they are accumulated

	acc0 = [acc_value for _ in range(256)] # 4 = h1
	acc1 = poly_mul(aa[:64], ss[:16], acc0,bs)
	acc2 = poly_mul(aa[64:128], ss[16:32], acc1,bs)
	acc3 = poly_mul(aa[128:], ss[32:], acc2,bs)

	return acc3

def A2A(A, R,  gamma,r):
# This is the bare minimal A2A functions used to verify the functionality of the designed AOverflowBit	
	m=1
	k=4
	C=0
	
	CA = {}

	for AA in range(0, 2**k):
		CA[AA] = gamma if AA + r < 2**k else (gamma + 1) % 2**10 #((AA + r) >> k) + gamma #gamma 
		
	Gamma = 0
	P = 0

	Gamma += gamma*2**(k)
	Gamma = Gamma % 2**(m+k)

	P = r
	
	for i in range(0,16):
		a_t = (A>>i*4)&0xf
		r_t = (R>>i*4)&0xf
		

		a_t = (a_t - P - Gamma) % 2**(m+k)

		Rl = r_t % 2**k
		a_t = (a_t + Rl) % 2**(m+k)
		Al = a_t % 2**k

		a_t = a_t >> k
		r_t = r_t >> k

		a_t = (a_t + CA[Al]) % 2**(m)
		C = (((a_t&0x1)<<i*4))|C
		
	return C



def parse(text):
# We use this to parse the output
    result = []
    for line in text.splitlines():
        if not line.strip():
            continue
        _, tail = line.strip().split(":")
        values = tail.split()
        for value in values:
            result.append(int(value, 16))
    #print(result)
    return result

memory = parse("""
0000:  383F765E270033A7  6D857AACB6201566  E6C2D077D327E014  F131CD03DBB839A4  CEB0F3D7C63C420C  1911FB75B202E3C3  BC88F3FCD6250907  87A8987F1F9C6770
 0008:  D7D10D771099CEF4  272D91E5DD849B5B  C222F83A932B3BB0  0D637976F0DEC601  A38DF0AFA5A6E3C3  9ECED620C72E601B  E06652DF571C7C32  BDE4878F6AD4C6AB
 0010:  54A3E39E2386F64A  83574986A76756F6  25CB110DE8A82BB3  10A1583BE4D8DD3A  CFBD3089D2FDD116  F07D171A78A6EBD8  3BB14FB5C78945E1  7C194DCE079FDC4B
 0018:  07EDDC9ACE6D3BB2  987FCA9538CDE48D  ACF81DFC9239E437  645C77F890968C54  893E3BFBECA3F195  2D432F7FEC057CB9  A8B52BA4EC55BF27  A67A9401B6BED3D0
 0020:  007D70692D8C9D9E  BEA7AC60F2F158D9  A5F1C23B74AF3286  16D54292C704A38F  08E0E6B2F669CC57  595A6EEAFA231D7B  5D4EF5FC5AA28864  878A1B5D1DE13D66
 0028:  92F7042B5B0B1059  0C7A97566E5BC101  CA8A292A4F29AA68  346090AA441576DF  9A61C930A3B09BBD  2A5E5E4CA01C5229  CE3CBC1CEE19DFC1  8B096ABA4C75D785
 0030:  8EAA29B8067E3603  73FB6968A3BEC033  6F9793DD02C5C044  11143D207D2B9610  B028E704C78E0B8A  50EB33FE4AF0C430  040BA649CE429A96  ADCF7E01DCE53388
 0038:  281ABEFFBDCAACFB  FBD7868FF22CEB10  D78A7BDE7C1275BF  778871A95E2E1F7C  8E105D0A3065AB46  CAE543D5245CC378  26EE444330F23B94  6A14C1E1F18EE2B9
 0040:  71D4E35445F6A103  00BBFCAEBB4D31B6  1266AFFCBFE321C2  9FA62A9EF2CA38EA  D6B699AE65983E48  8C063F3FDD836D56  492F278B3D9A3115  7F27C8832F20747C
 0048:  57B418BB61F2FC2A  18B902C19B9A3CDA  7C06CE82702F83E5  69CAA88F7BF6DC99  BC56672DA854C658  A04772E7AE2993A8  FB5FB3617532D68B  37CCB00F3747C67E
 0050:  2ED7F7CB2AB49EA0  E0C48601BCD72C54  DB788C75D923D644  2BB2C3DB088D95FA  A08B68B1DC8DFAE6  999E6791386E7B51  E2A07A0C95BD416F  27819DA9D45A30CB
 0058:  B6FA7284010DD222  85DDBF71B797B300  A359C0D13E6B45FE  7137FA02C982B27B  FC7790428F9F2C60  BE9030A49378BD54  4B8165C939048C37  864CA7B6873E7F4E
 0060:  BA117A0445C595AA  F6BD18960661ED93  5A48F676B5D222C7  6B1D3D176A3A9CAD  BC822189F36C6D50  999E0B7807226619  AB1864BD316D10D9  3EF76DB525F8491C
 0068:  8308C5B09FB38223  D50C1CC5EC27C7FC  2BC58F81938951A7  8278196CF3DA8187  CC80A828B73EFEA4  B89E5F4E619D1C68  4D3E1C1850AB3BEB  EE201C6FBB7C3198
 0070:  9E1D911C0F2C068C  FB5FDA8AC10FF9FE  B165EDBE17C40917  1CD311066C069AC4  2919887580748FF2  20BDF6EC71FB3957  11741D807C541553  8444660B88009488
 0078:  41106250338C385E  B78F7DE3A18BFC7B  E94F61BD22836A08  B37D45DA1CAD9105  A23AE1E85466FADE  6C365ADBC45F7AE7  E3C98A66C07C6085  955CA06C8ED7AA92
 0080:  CAF0CC6F3849BB8E  D21E96FF109658D6  04FC7E99C42244E4  8F10B0FE8428307F  20D5A0F2FC185463  DA0432CE84F8186F  EF7077ED6574761F  BA47EC3EB9B45A36
 0088:  CF76578D34F395AF  ED2E28AFB0DC99DB  1B4334BD33A906C9  D5BE22E520A8D66D  70E67F903234C3C2  5DBA96956BD58B2E  A48A5A29451EDFF3  19EA8742E63F7CFA
 0090:  10E06BA9722E2DA0  764145748014FEC3  5AF31DD1FCC59DD7  DF408F47542ADB8E  179B9C8D2DBE7003  A87312078F45D6B9  E2C32B76A5F5CEE5  CABEA2F85E58586E
 0098:  267D8829D44C7749  2BDFD195AC76C93E  457BA857FE5B9046  B47F374AB9B7DC1C  C7EDC176B81B3CBE  06E6112544DC6B78  F844B555DAB75E69  BE3D44F4F9C564B6
 00A0:  C880644745384B43  0FC7A81867AEA5A3  4835ADCE48CC0D89  B0DA0FF88A562593  C8FCEBBDBDA66828  1CDC1C11D8832CB6  03DA3DF0F389E757  873CC6F5D3C3CC2C
 00A8:  29AE44F047998E19  BE55376BF41227B2  12E4A75443EA1347  79917C7E63621040  C3345A164A93E3FF  BB74A41FC44F46DC  08FA078C8E7BAAE5  BE1785469E6E5B03
 00B0:  ACB15A1BF92788CC  3593C02AA26A05BF  4300975D9A95A2EE  85711A5C6D693CE7  D8CC0E267076B64C  30AC989293190778  A24B1CF6BFF97B92  5177F5425337548E
 00B8:  B6FECB4424FDF487  16EC9A26CEBD0B76  AD80E41BA22E0DF4  13E921F109AE32BF  285A34EB7357C81F  07C99DF20BEF4FC1  E048052E39C13F4C  3A1D47CA893A4785
 00C0:  44E11B0D826A788E  8B2EA1D4087A1EDF  347B8FA3E3A6C789  41E36CA8D1F7716E  33FEF5DF7B9B8D91  6DF5B17B99B548C6  DEC374EF5E2122C2  B858A281E2DF188C
 00C8:  EFABCA34DF97901F  061C8E0926CD76AE  C7F1E74726868F53  24E717000D52E13C  350937D5206D89B5  6B4D401646663E77  CCC1F99B724D1F92  52CC94975ADA1AF2
 00D0:  99FE374887F1F683  273D295EBCB80275  C880644745384B43  0FC7A81867AEA5A3  4835ADCE48CC0D89  B0DA0FF88A562593  C8FCEBBDBDA66828  1CDC1C11D8832CB6
 00D8:  03DA3DF0F389E757  873CC6F5D3C3CC2C  29AE44F047998E19  BE55376BF41227B2  12E4A75443EA1347  79917C7E63621040  C3345A164A93E3FF  BB74A41FC44F46DC
 00E0:  08FA078C8E7BAAE5  BE1785469E6E5B03  ACB15A1BF92788CC  3593C02AA26A05BF  4300975D9A95A2EE  85711A5C6D693CE7  D8CC0E267076B64C  30AC989293190778
 00E8:  A24B1CF6BFF97B92  5177F5425337548E  B6FECB4424FDF487  16EC9A26CEBD0B76  AD80E41BA22E0DF4  13E921F109AE32BF  285A34EB7357C81F  07C99DF20BEF4FC1
 00F0:  E048052E39C13F4C  3A1D47CA893A4785  44E11B0D826A788E  8B2EA1D4087A1EDF  347B8FA3E3A6C789  41E36CA8D1F7716E  33FEF5DF7B9B8D91  6DF5B17B99B548C6
 00F8:  DEC374EF5E2122C2  B858A281E2DF188C  EFABCA34DF97901F  061C8E0926CD76AE  C7F1E74726868F53  24E717000D52E13C  350937D5206D89B5  6B4D401646663E77
 0100:  CCC1F99B724D1F92  52CC94975ADA1AF2  DCB3912DD56C7E5F  251B5C2E584987A5  0E216DDEA73C0F0E  92E05D87EF5EB3B4  BF82077DBE6A6216  7EE98532B643B436
 0108:  3DA1707BCD959734  565A905A144892A9  071D7610BBB25AB0  C512F0AE32454F4B  297C485C451C8CB3  9439E71B99EBB7E6  D4212264BFF98EE2  1F7902A42C1FF84D
 0110:  347B389484A67102  6E53246EFA74B9E1  772DB16750A2D5EE  A9502BF773A345EC  23B6EF746573F4AF  14AE60969352E25E  51BB00731D09D3E8  5423630A91D93C25
 0118:  96D6D7022EED229B  6D6E87E0ADA67FE2  74EDA026AE6D7FA6  01247E9A5680D50C  6914C80619CD6E74  9132ACC467B2273C  AE4B600B8C6061FD  01A99FF300476057
 0120:  792D9A7C0364724E  156E2E5580585277  36D0ACFEF012B9C0  371FB211EF0EF4CA  0518CD0E29D5C54F  FE38B590C4857578  288C4B43D5F1713C  3595017129FDC836
 0128:  80BF0F2BF46E58B8  C9D5120546C43AF1  7BB85AB3F466CE73  7E3CFFABB6F4EEB9  807360B69B91438D  6E0AF04E7FDD4B57  2B415C27952B757E  5A3A403D803AB2C9
 0130:  87340B27E92A6403  80CD3D347B135EA0  6AD2502EF6EF506C  D85380AACE985CCB  B66D08AE77BCD055  A4FB7F08B583C4C0  0E216DDEA73C0F0E  92E05D87EF5EB3B4
 0138:  BF82077DBE6A6216  7EE98532B643B436  3DA1707BCD959734  565A905A144892A9  071D7610BBB25AB0  C512F0AE32454F4B  297C485C451C8CB3  9439E71B99EBB7E6
 0140:  D4212264BFF98EE2  1F7902A42C1FF84D  347B389484A67102  6E53246EFA74B9E1  772DB16750A2D5EE  A9502BF773A345EC  23B6EF746573F4AF  14AE60969352E25E
 0148:  51BB00731D09D3E8  5423630A91D93C25  96D6D7022EED229B  6D6E87E0ADA67FE2  74EDA026AE6D7FA6  01247E9A5680D50C  6914C80619CD6E74  9132ACC467B2273C
 0150:  AE4B600B8C6061FD  01A99FF300476057  792D9A7C0364724E  156E2E5580585277  36D0ACFEF012B9C0  371FB211EF0EF4CA  0518CD0E29D5C54F  FE38B590C4857578
 0158:  288C4B43D5F1713C  3595017129FDC836  80BF0F2BF46E58B8  C9D5120546C43AF1  7BB85AB3F466CE73  7E3CFFABB6F4EEB9  807360B69B91438D  6E0AF04E7FDD4B57
 0160:  2B415C27952B757E  5A3A403D803AB2C9  87340B27E92A6403  80CD3D347B135EA0  6AD2502EF6EF506C  D85380AACE985CCB  965A4FFDDD340ABE  29EB22AF0CBA3106
 0168:  1AE74E3418523F9D  11BC81856FF6C3FB  B8A6C36028DA980C  F5BDDD8BA82A963B  37E6A05257C8670E  9AEB4967BB732E73  8D5073A3336906FF  B09171C926F712B4
 0170:  24082E628C0D9189  6F8C737CA31A918D  3ECF8A4E698663E0  EEDD27C8A5D61EFC  FEC21ECC9C81BCBE  AE56DD049D7B36B6  CF33F4AB925DFACC  2DCE21E205B5C0C0
 0178:  BE4CB4311E6F926B  F77FDC268EF44EE1  8BAE42FE78CA3A5A  8C3787892CAFA21E  4D8C4A23DA689358  66EF3052CBB14A43  A95797848615C3D5  FF965E77C24582C6
 0180:  4D510F17662C5CAB  2FE5D55D4D66D13F  23A5122ED43D07E5  2BA1C1F33C87C137  8EB3CD3ED31C53F8  71126DAA9C9310E9  8FB986101CB43CAB  234834CB5107494C
 0188:  DA534DF655F11C57  CC0965CCE34901D8  59C6A94A9C4F46AE  82C6F7B04A957953  B8F932FCC76E08BB  12DF15F8D24DD30E  F6B1FD27B612875E  6FEF9412F83AD433
 0190:  413ED0ECD6DC6C1F  5A53673E31FCB321  B7F8DF06B070FA97  9D65E325A8F69AEE  0156116EE579F4B5  64A5968747C3A475  050FCAAB2A4DE40C  11011BCCB43941DD
 0198:  B398820FF3A73EB6  4DEC7208BA7B460F  61273433474878EB  4F1BA905E76C56CF  A534ABB0EB1849BB  B7517B26FD46EE71  FBE3F77089AA25BC  7CC4B5ECE573EE4A
 01A0:  A937CF0CE3D919B3  F35A060C8EC61F1D  0A6BDE86385DEA45  F7C4A8CBCE930A8C  DA437AA5E4740ABB  99046C03D48E8F66  D7E7734323923FFE  C4B6BFAB976E3D2F
 01A8:  C62C8BC6229A3BA7  71A54A0D3CD4EA58  FAD5F63BB0290937  70CD071ADFB55A7B  9F24068FCF3CA58F  A380A18AB8F04C14  2852CA2FE25AEF6C  29734B0D2FB53F1A
 01B0:  EA4A16B0BD01A25A  32F285B3E4C88BF6  45B51BA1DBBCCC96  0B0EFD8D711DA3BD  0A2D4C804C47A0A7  42F9FC436B80D144  CF63B69834F1EAF5  A6104A1F41FE11CB
 01B8:  0A4E02D50F2C1898  0ED4055714D414D7  0D3C07991C9A10F7  0E010A321E5716F0  0E56196B0874118B  12301FFA10EC03A1  011C01B10CC31D2F  12F31FCD19830F7B
 01C0:  10871661083A1B5B  1092132D06AA17E8  17F91A2F160F0B2E  001B112900320621  029D0DFE00F50DAE  172706F21D300A15  1CF412DF14BC195A  16CE19481AAB0F7C
 01C8:  177F032F14C8019A  1FAE1C221981047B  04140AD912621366  041D136D0CF717CD  0C1C0B5002D4011C  089419D103C50757  0E04139318BC0914  07020523036D1369
 01D0:  07FD0BDD0EA90C60  1CD7056314D50DB9  12E914301E911D48  1CB21E850E4A1812  042C0A7F14F918BE  05100D8A1B830B56  1A2A0BB90E740828  09EA0D83138900D5
 01D8:  04E411D3172D1133  09C2072A080B0497  1EFF18720F5C0AFB  191E04A3092A06D6  0CDD1DAE1A8F1536  06C00D1E0F8104B9  02720D2F04DF12C4  16F3089B0A7C1067
 01E0:  010A198F0A90171E  02791FB2083512E6  1262166217920F64  08D2098905BF0577  070B068206CC16CA  0C06146804B710AF  15A513620F1205DE  12E606851B3A0B73
 01E8:  14070CC217840FC7  076D0FF20754143E  1D9B1FDA040A0EAE  039F0F391A790916  171D1F550E130051  03A917AC11B31638  143E1DC219D30C59  01B6101D06A41532
 01F0:  1C7D1572174E1F94  14FC16141F41143F  12BD110B16F11AD0  131613D705C40691  10C705FA16CA10C1  091E00C21ED61038  0AD01A9A058702E9  12AA099E0FF11EAA
 01F8:  094C061814CF17EA  1F6716A30FD40398  16C90AAE0FFB0042  11670908167612B0  18990AF716951624  1B49164D052E1D81  0AE91A650A1207C1  06FB1FA60900155F
 0200:  1D3D1DD41BAD0352  179D0CA912BA0D9E  11111E1607301C99  0C090D3D0B791FAD  057316E310FC0BD9  149B157E14240150  0B551EA3018711F7  07CB10A8026C00FC
 0208:  0CA91CF90982038E  03170ABB10EE1790  05E50F940BF30958  14471D481A1D18A7  0C641355047A08EB  04D709021E7E07A0  0A370BD219A117EB  119303A613080BC1
 0210:  1F73137E09170BB0  123F073F178C10AE  1545101314F31141  04AE14FD05D91719  1DF5094016B708E4  0E1913B51B8C0F42  065A080B112618CB  109F19211232199A
 0218:  033A04A61EA402CD  0B5B164D04950432  1EAA14850DD005EA  17131ABA11500CFF  0CE7017B1F980BB4  1FA011F60C230641  0F4F1C2D003F1294  0C7F0B770DAE08B0
 0220:  0B9F0E3004D30E5B  112E11F31334027B  01DA1C831077045F  0983165F11A50A62  025D0BF9088711E3  0D9E1926113E13F6  14A714C309BB18E3  0A5518AE0B861D27
 0228:  0C05098C11581B3F  1CAB0C1A1D650F73  0CB70683069B0217  19380C34073F1BB0  06BE001018311B66  1F9C0FB10FE8053E  151C1FB619FF072F  06D918CC1D650C9C
 0230:  18971A341EB215C7  1E7B0A57152C1DCC  073F1AB90FFB12D2  108F1A7F169B1AFA  0F21082818AF0A3B  0D1F01CA15730C91  1135097B0845126A  0C3D0C631BBE1F86
 0238:  1E201F580CA011E8  19480538011807E0  120801A818281698  137011180D280198  05B0091005F00538  06600D000F680978  1D20175818D00A50  0F10183013101A18
 0240:  06981CF01C881AC8  16681A980C1811C0  0CF81A0807C00E50  03B01E301E980610  07A019A01C880510  0BF00BA81C401860  1BC811000DB01568  0498193809C81C68
 0248:  1058121800280050  1DE01A20117819F8  02B807B00A1012C0  0548180018B80870  0A98060003C81190  016003200EB80288  06D00738188819C0  1D701D4008580E50
 0250:  0FA01C281BA01690  06D812680A980BF8  1628006810B00548  18901B0819A003D8  00181E28181006D8  0F581C6807F00278  1CB00C0015C01C80  084019E002400D18
 0258:  0F801B6818481530  179803A81BB00A48  09280AE814100B00  0B5801E017600290  1F6813F004D009E0  151812701C780DA0  011004E01B300160  1D600CD81F780240
 0260:  121811681C200860  094008201C981A40  13B81D200E2801B8  1FF813C005480C28  09E80AA007C01690  1E7019800A680A28  13C0156816B01470  0918158805D00758
 0268:  1B480C000A881468  04301B1811C81C38  1FE00D7815001698  192013F013381388  1EE8169016881F98  04480AA808481988  05D019E81A401728  06601A9006180B20
 0270:  0670194018F01CF8  1728041814B81158  143813801B881790  1EE01F30001812C8  0F301F901CD00F80  0C3814880BD80698  12E80C7005980BB8  00001FC01AC007B8

""")

atemp_0 = [memory[i+2] for i in range(0,156)]
a0 = [0 for i in range(0,192)]

concat=0

# Unpack the public polynomial coefficients from the memory storage
for i in range(0,192):
	if i<156:
		concat = (atemp_0[i]<<(i*64-i*52)) | concat
	a0[i] =  (((concat>>39)&0x1fff)<<48)|(((concat>>26)&0x1fff)<<32)|(((concat>>13)&0x1fff)<<16)| (concat&0x1fff)
	concat=concat>>52


# polynomial multiplications
as0   = [memory[i] for i in range(160,208)]
as1   = [memory[i] for i in range(260,308)] 
Gamma = (memory[49]>>48)&0x1
r     = (memory[49]>>16)&0xf

# three-way A2A
f1 = [A2A(as0[i], as1[i], Gamma,r) for i in range(0,48)]

	
mul0_g = vector_mul(a0, as0, 4)
mul1_g = vector_mul(a0, as1, 0)
mul2_g = vector_mul(a0, f1, 0,True)
mul2_g = [(2**13 - x)&0x1fff for x in mul2_g] # in hw this is done together with the multiplication


"""
#for verifying A2A
f11 = [A2A(as0[i], as1[i], Gamma,r) for i in range(0,48)]

final_a2a1 = [((f11[4*i+3]<<48)|(f11[4*i+2]<<32)|(f11[4*i+1]<<16)|(f11[4*i])) for i in range(0,12)]

for i in range(12):
	if final_a2a1[i]!=memory[i+410] :#or final_a2a2[i]!=memory[i+424]:
		print("ERROR ",hex(final_a2a1[i]),hex(memory[i+410]))
		break
else:
	print("All good :)")#,hex(final_a2a1[i]),hex(memory[i+410]),hex(final_a2a2[i]),hex(memory[i+425]))
"""

# for verifying A2A and polynomial vector multiplication.
# we are checking if the result in memoryis same as the result we get here for input variable stored in the memory.


for i in range(64):
	if ((mul0_g[4*i+3]<<48)|(mul0_g[4*i+2]<<32)|(mul0_g[4*i+1]<<16)|(mul0_g[4*i]))!=memory[i+440]:
		print("ERROR s1",i,hex(((mul0_g[4*i+3]<<48)|(mul0_g[4*i+2]<<32)|(mul0_g[4*i+1]<<16)|(mul0_g[4*i]))),hex(memory[i+440]))
		break
	if ((mul1_g[4*i+3]<<48)|(mul1_g[4*i+2]<<32)|(mul1_g[4*i+1]<<16)|(mul1_g[4*i]))!=memory[i+504]:
		print("ERROR s2",i,hex(((mul1_g[4*i+3]<<48)|(mul1_g[4*i+2]<<32)|(mul1_g[4*i+1]<<16)|(mul1_g[4*i]))),hex(memory[i+504]))
		break
	if ((mul2_g[4*i+3]<<48)|(mul2_g[4*i+2]<<32)|(mul2_g[4*i+1]<<16)|(mul2_g[4*i]))!=memory[i+568]:
		print("ERROR c",i,hex(((mul2_g[4*i+3]<<48)|(mul2_g[4*i+2]<<32)|(mul2_g[4*i+1]<<16)|(mul2_g[4*i]))),hex(memory[i+568]))
		break		
else:
	print("All good :)")
