# Using Kavach artifacts 

The Kavach team welcomes you to the usage guide for the artifacts of our paper!  


The process is divided into four major straightforward steps: design synthesis,
FPGA test, measurement and finally T-test.
While explaining these steps, we will also explain where you can find the
required files. In this file, we only give simple usage instructions. To get a
description of how these files work, please refer to the last section of this
file. There we list the readme files that provide a deeper explanation of the
scripts we use here.

*If you only want to run T-tests on our dataset, go directly to the last step.*


After downloading the entire repository, go to the directory `kavach_artifacts/` and open a terminal here. Now, please follow the steps given below.

## Step 0: python dependencies

Step 1, 2 and 3 below depend on the following python packages:
- chipwhisperer
- numpy
- picosdk
and Step 4 depends on:
- matplotlib
- numpy
- scalib

We recomment using a python virtual environment to reproduce our setup (known to be working with python3.10):
```
$ python -m venv ve
$ source ve/bin/activate # See https://docs.python.org/3/tutorial/venv.html if not in linux or Mac OS + bash.
$ pip install -r requirements.txt
```


## Step 1: Generating the bitfile and programming the FPGA.

### Bitfile generation

*Remark: you may skip this step and use our pre-generated bitfile `HW/cw305_top.bit`.*

Here are the instructions for running the synthesis for the CW305 board.

First, create the vivado project using:

```
 $ cd ./HW/
 $ source <directory where vivado is present>/Xilinx/Vivado/2019.1/settings64.sh  //e.g. /opt/Xilinx/Vivado/2019.1/settings64.sh
 $ vivado
 $ vivado_tcl_console: source kavach.tcl
```

This tcl script is tested for Vivado2019.1. We also provide a new tcl script
`kavach_2022.2.tcl` as some property names change between these two vivado
versions.

Now Vivado will create a project folder named `kavach` which is ready to use.
This project can be opened from the Vivado GUI to generate the bitstream and
program an FPGA.

The design provided is implemented on Xilinx Artix-7 xc7a100tftg256-3690 FPGA.


## Step 2: Running the desired instructions and verifying the result 

Now, the FPGA is programmed with the bit file (.bit) generated for the design.
We do not need Vivado anymore, so we exited the command mode. Next, we would
want to run the instructions for polynomial multiplication and collect the
measurement. But before this, it would be great if we could verify that the
instructions run correctly and give the desired result.

First, we send the instructions that we want to perform. For this, we run the
script `SW/crypto_setup_masked_*design.py`. Within the script, we have support for the
instructions to be run multiple times. You can also do this (in case you
skipped STEP 1), along-with programming the FPGA with the bitstream file
specified using the following command:

```

$ python3 ./SW/crypto_setup_masked_design.py  --bitstream-file ./HW/cw305_top.bit
```

To see the results in the FPGA memory, we perform a memory dump by running the following command.
``` 
$ python3 ./SW/crypto_memory_dump.py
```

After this, in the terminal, you will see a memory dump. Just copy this and
use it to replace the memory dump in `SW/verify_masked_design.py` [Lines 108-186].
Now run the following command.
``` 
$ python3 ./SW/verify_masked_design.py
```
The file `SW/verify_masked_design.py` has the code that reads the polynomials
from the memory dump, performs the A2A and vector multiplication and
accumulation, and then checks if the result in memory is the same as expected.
Now that we have verified the functionality, we can proceed with collecting the
measurements. 



### Note:

It has been observed that the board stops working if the `./SW/crypto_setup_masked_*design.py` scripts are called more than once without power-cycling. Thus, in case this problem appears please use power-cycling.

## Step 3: Measurement Collection 

For measurements, we use PicoScope6000 configured with timebase 4, and choose a
FPGA clock frequency of 10MHz. This gives us 31.25 samples per clock cycle. Run
the snippet below to collect these measurements. It collects 500 measurements and stores
them in `data/raw_data` folder.

```
$ python3 SW/store_raw_data_of_measurement.py raw_data --setup-script SW/crypto_setup_masked_design.py --teardown-script SW/crypto_memory_dump.py --clock-frequency 10_000_000 --measurements 500 --timebase 4  --python-exec $(which python3)
```

These measurements collect samples for the entire run including initialization,
etc. However, we are only interested in the execution of AOverflowBit and
Polynomial vector multiplication. For this, we trim the waveform a bit. This is
done using the script `data/trim_traces.py`.

```
 $ # /!\ Edit the script to configure which files and what you want to trim.
 $ # /!\ See comments in the script.
 $ # /!\ See also `data/Readme.md`.
 $ python3 data/trim_traces.py
```

 Now that we have collected enough measurements, we will go to the next step and see how to process it to get the t-test.


## Step 4: T-test evaluation

### Option 1: Use our dataset

*We recommend you start with this, before moving to your own measurements.*

Download our dataset (see `data/Readme.md`) and unzip it to get directories
`data/kavach_datasets/masked` and `data/kavach_datasets/masked_flawed` that
contain `.npy` files.

Then, run
```
$ python3 data/t_test.py
```
 
The result gives the number of traces processed, the maximum and minimum t-value, and a plot with lines drawn to mark the +-4.5 boundary.

### Option 2: use your traces

You may edit the `datasets` variable in `data/t_tests.py` to point to your trimmed dataset(s), then run the script.


With this, we conclude our little journey for verifying the proposed masking technique via t-test.



## For understanding the codes

To understand how the Software and Hardware codes help us perform this evaluation, please refer to the following READMEs:

- `SW/README.md`
- `HW/README.md`
- `data/README.md`

## Board connections

We made the following connections between to collect the results:


<div align="center"><img src="./connections.jpg"  width="420" height="420"></div>



1. A USB connection is used to communicate with software components. 
2. A JTAG programmer is used to flash the FPGA. 
3. Pin B14 used for Trigger connected to Channel B of the Picoscope. 
4. SMA connector to transfer power consumption to PicoScope; Connect the SMA cable from the Picoscope Channel A to X4 (amplified shunt output) on the CW305.
5. SMA connector for clocking the FPGA. Connect Picoscope with PLL channel x6 to the AUX input.  








 
