# Copyright TU Graz, Austria <sujoy.sinharoy@iaik.tugraz.at>.
#
# This file is part of Kavach <https://extgit.iaik.tugraz.at/sesys/kavach_artifacts>.
#
# Kavach is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License version 3 as published by the Free
# Software Foundation.
#
# Kavach is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Kavach. If not, see <https://www.gnu.org/licenses/gpl-3.0.txt>.

# This file performs the t-test on the measurements stored in the folder ./kavach_datasets/masked*/

import matplotlib.pyplot as plt
import numpy as np
import tqdm

from scalib.metrics import Ttest

import numpy as np

TRACE_LENGTH = 93031


# (dataset name, dataset path template, number of set files)
datasets = [
        ('masked', 'data/kavach_datasets/masked/set{}_{}.npy', 90),
        ('masked_flawed', 'data/kavach_datasets/masked_flawed/set{}_{}.npy', 1),
        ]

for dataset_name, dataset_path, num in datasets:
    ttest = Ttest(TRACE_LENGTH, d=1)
    print("Ttest for", dataset_name)
    ntraces = 0
    for set_idx in tqdm.trange(0,num):
        traces1 = np.load(dataset_path.format(1, set_idx))
        traces2 = np.load(dataset_path.format(2, set_idx))

        ntraces = ntraces + traces1.shape[0] + traces2.shape[1]
        
        X1 = np.zeros((traces1.shape[0],), dtype=np.uint16)
        X2 = np.ones((traces2.shape[0],), dtype=np.uint16)

        ttest.fit_u(traces1,X1)
        ttest.fit_u(traces2,X2)

    t = ttest.get_ttest()

    print("\tTotal umber of traces:",ntraces)
    print("\tMaximum and minimum t-value :",np.amax(t),np.amin(t))
    plt.figure()
    plt.title(f"Ttest for dataset '{dataset_name}'.")
    plt.plot(np.array([i for i in range(len(t[0]))]),np.array([4.5 for i in range(len(t[0]))]))
    plt.plot(np.array([i for i in range(len(t[0]))]),np.array([-4.5 for i in range(len(t[0]))]))
    plt.plot(np.array([i for i in range(len(t[0]))]),np.array(t[0]))
plt.show()


