# Copyright TU Graz, Austria <sujoy.sinharoy@iaik.tugraz.at>.
#
# This file is part of Kavach <https://extgit.iaik.tugraz.at/sesys/kavach_artifacts>.
#
# Kavach is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License version 3 as published by the Free
# Software Foundation.
#
# Kavach is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Kavach. If not, see <https://www.gnu.org/licenses/gpl-3.0.txt>.

import os
import sys
import struct
import logging
import datetime

#This file is used to visualize raw plots and also trim the measurements

import argparse
import numpy as np
from matplotlib import pyplot as plt
 
PLOT_RAW_DATA = 0                                     # This variable is set to 1 to visualize the raw measurement. When set to 0 it trims the measurement.

directory = "./"                                     #This is the path to the data. Right now it assumes data is in the same folder as this file. Please MODIFY it if this is not the case.



S_per_C= 31.25
INS_1=2977                                            # Length of AOverflowBit + polynomial vector multiplications (total 9 polynomial multiplications)
i_per_m = 23                                            # number of iterations in every measurement
INS_LEN = int((2*INS_1+2*50+5*2006+30+40+15)*S_per_C) # This includes the waits given in between different meausrements to avoid noise propagation
            
def find_01(value): #function to check trigger change from 0 to 1
    v_len = len(value)
    index = []
    for i in range(v_len-1):
        if value[i]==0 and value[i+1]==1:
            index.append(i+1)
            i = i+1
    return index


def find_10(value): #function to check trigger change from 1 to 0
    v_len = len(value)
    index = []
    for i in range(v_len-1):
        if value[i]==1 and value[i+1]==0:
            index.append(i)
            i = i+1
    return index


def read_mbin(filepath): #A routine to read a file in MBIN format into a numpy ndarray.
    with open(filepath, 'rb') as fd:
        buf = fd.read(32)
        header = struct.unpack('<bbbbbbbbbbbbIIIII', buf)
        assert bytes(header[0:10]) == b'matrix-f16'
        height, width = header[-2], header[-1]
        array = np.ndarray((height, width), dtype=np.float16)

        for r in range(height):
            for c in range(width):
                item = struct.unpack('<e', fd.read(2))
                array[r, c] = np.float16(item)
    return array




if PLOT_RAW_DATA:
# This function plots the first operation run of one measurement. These can be changed if needed. 

    for mnum in range(0,1):
        mnums = '{:03}'.format(mnum)
        mnums_bin = directory+"measurement"+mnums+".mbin"
        mnums_mat = "set"+mnums+".mat"

        a = read_mbin(mnums_bin)
        print("file read:", mnums_bin)

        voltage000 = a[0]
        trigger000 = a[1]

        trigger000 = np.array([1 if x>200 else 0 for x in trigger000]) # set the trigger values to 0 or 1 depending on the amplitude threshold

        index_01 = find_01(trigger000) # find the indices when trigger goes up
        index_10 = find_10(trigger000) # find the indices when trigger comes down

    # Plot the raw measurement
    fig, axs = plt.subplots(2)
    axs[0].plot(trigger000[index_01[0]-40:index_01[0]+2*INS_LEN+40],'r')  # plot for trigger
    axs[1].plot(voltage000[index_01[0]-40:index_01[0]+2*INS_LEN+40])      # plot for voltage/raw_measurement
    plt.show()
    
else:
# This function currently trims 2 measurement files, this can be modified if needed.
    
    index_s = 0 # measurement starting index
    index_e = 1 # measurement ending index

    
    
    allv=[]
    for mnum in range(index_s,index_e+1):
        mnums = '{:03}'.format(mnum)
        mnums_bin = directory+"measurement"+mnums+".mbin"
        mnums_mat = "set"+mnums+".mat"

        a = read_mbin(mnums_bin)
        print("file read:", mnums_bin)

        voltage000 = a[0]
        trigger000 = a[1]

        trigger000 = np.array([1 if x>200 else 0 for x in trigger000]) # set the trigger values to 0 or 1 depending on the amplitude threshold

        index_01 = find_01(trigger000) # find the indices when trigger goes up
        index_10 = find_10(trigger000) # find the indices when trigger comes down
	
	#trimming the measurement according to the instruction length
	#Values 2006 and 8005 come from the hardcoded wait that is given after every trigger
        for i in range(len(index_10)):
            allv.append(voltage000[index_01[i]+int((2006+8005)*S_per_C):index_01[i]+INS_LEN+int((2006+8005)*S_per_C)])

    set1=[]
    set2=[]
    
    # Add the measurements the ste 1 and set2 alternately from the trimmed measurement
    for i in range(i_per_m*(index_e-index_s)):#3 for 3 data sets collected
            set2.append(allv[i][0:int(INS_1*S_per_C)])
            set1.append(allv[i][int((INS_1+5*2006+100+30+40+15)*S_per_C):INS_LEN]) 
            # The fixed values here come from the wait given in between the two instructions

    # save results
    s1 = np.asarray(set1, dtype=np.int16)
    np.save("{0}{1}.npy".format("set1_", 0), s1)

    s2 = np.asarray(set2, dtype=np.int16)
    np.save("{0}{1}.npy".format("set2_", 0), s2)


